/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 * @format
 */
'use strict';

exports.__esModule = true;
exports.default = usePressability;

var _Pressability = _interopRequireDefault(require("./Pressability"));

var _react = require("react");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function usePressability(config) {
  var pressabilityRef = (0, _react.useRef)(null);

  if (pressabilityRef.current == null) {
    pressabilityRef.current = new _Pressability.default(config);
  }

  var pressability = pressabilityRef.current; // On the initial mount, this is a no-op. On updates, `pressability` will be
  // re-configured to use the new configuration.

  (0, _react.useEffect)(function () {
    pressability.configure(config);
  }, [config, pressability]); // On unmount, reset pending state and timers inside `pressability`. This is
  // a separate effect because we do not want to reset when `config` changes.

  (0, _react.useEffect)(function () {
    return function () {
      pressability.reset();
    };
  }, [pressability]);
  return pressability.getEventHandlers();
}

module.exports = exports.default;