"use strict";

exports.__esModule = true;
exports.default = void 0;

var _createElement = _interopRequireDefault(require("../createElement"));

var _css = _interopRequireDefault(require("../StyleSheet/css"));

var _setAndForwardRef = _interopRequireDefault(require("../../modules/setAndForwardRef"));

var _useElementLayout = _interopRequireDefault(require("../../hooks/useElementLayout"));

var _usePlatformMethods = _interopRequireDefault(require("../../hooks/usePlatformMethods"));

var _useResponderEvents = _interopRequireDefault(require("../../hooks/useResponderEvents"));

var _react = _interopRequireWildcard(require("react"));

var _StyleSheet = _interopRequireDefault(require("../StyleSheet"));

var _TextAncestorContext = _interopRequireDefault(require("./TextAncestorContext"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) Nicolas Gallagher.
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var Text = (0, _react.forwardRef)(function (props, ref) {
  var accessibilityLabel = props.accessibilityLabel,
      accessibilityLiveRegion = props.accessibilityLiveRegion,
      accessibilityRelationship = props.accessibilityRelationship,
      accessibilityRole = props.accessibilityRole,
      accessibilityState = props.accessibilityState,
      children = props.children,
      dir = props.dir,
      forwardedRef = props.forwardedRef,
      importantForAccessibility = props.importantForAccessibility,
      nativeID = props.nativeID,
      numberOfLines = props.numberOfLines,
      onBlur = props.onBlur,
      onClick = props.onClick,
      onContextMenu = props.onContextMenu,
      onFocus = props.onFocus,
      onLayout = props.onLayout,
      onPress = props.onPress,
      onMoveShouldSetResponder = props.onMoveShouldSetResponder,
      onMoveShouldSetResponderCapture = props.onMoveShouldSetResponderCapture,
      onResponderEnd = props.onResponderEnd,
      onResponderGrant = props.onResponderGrant,
      onResponderMove = props.onResponderMove,
      onResponderReject = props.onResponderReject,
      onResponderRelease = props.onResponderRelease,
      onResponderStart = props.onResponderStart,
      onResponderTerminate = props.onResponderTerminate,
      onResponderTerminationRequest = props.onResponderTerminationRequest,
      onScrollShouldSetResponder = props.onScrollShouldSetResponder,
      onScrollShouldSetResponderCapture = props.onScrollShouldSetResponderCapture,
      onSelectionChangeShouldSetResponder = props.onSelectionChangeShouldSetResponder,
      onSelectionChangeShouldSetResponderCapture = props.onSelectionChangeShouldSetResponderCapture,
      onStartShouldSetResponder = props.onStartShouldSetResponder,
      onStartShouldSetResponderCapture = props.onStartShouldSetResponderCapture,
      selectable = props.selectable,
      testID = props.testID,
      onMouseDown = props.onMouseDown,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      onMouseMove = props.onMouseMove,
      onMouseOver = props.onMouseOver,
      onMouseOut = props.onMouseOut,
      onMouseUp = props.onMouseUp,
      onTouchCancel = props.onTouchCancel,
      onTouchCancelCapture = props.onTouchCancelCapture,
      onTouchEnd = props.onTouchEnd,
      onTouchEndCapture = props.onTouchEndCapture,
      onTouchMove = props.onTouchMove,
      onTouchMoveCapture = props.onTouchMoveCapture,
      onTouchStart = props.onTouchStart,
      onTouchStartCapture = props.onTouchStartCapture,
      href = props.href,
      itemID = props.itemID,
      itemRef = props.itemRef,
      itemProp = props.itemProp,
      itemScope = props.itemScope,
      itemType = props.itemType,
      rel = props.rel,
      target = props.target,
      unstable_ariaSet = props.unstable_ariaSet,
      unstable_dataSet = props.unstable_dataSet;
  var hasTextAncestor = (0, _react.useContext)(_TextAncestorContext.default);
  var hostRef = (0, _react.useRef)(null);
  var setRef = (0, _setAndForwardRef.default)({
    getForwardedRef: function getForwardedRef() {
      return forwardedRef;
    },
    setLocalRef: function setLocalRef(c) {
      hostRef.current = c;
    }
  });
  var classList = [classes.text, hasTextAncestor === true && classes.textHasAncestor, numberOfLines === 1 && classes.textOneLine, numberOfLines != null && numberOfLines > 1 && classes.textMultiLine];
  var style = [props.style, numberOfLines != null && numberOfLines > 1 && {
    WebkitLineClamp: numberOfLines
  }, selectable === false && styles.notSelectable, onPress && styles.pressable];
  (0, _useElementLayout.default)(hostRef, onLayout);
  (0, _usePlatformMethods.default)(hostRef, ref, classList, style);
  (0, _useResponderEvents.default)(hostRef, {
    onMoveShouldSetResponder: onMoveShouldSetResponder,
    onMoveShouldSetResponderCapture: onMoveShouldSetResponderCapture,
    onResponderEnd: onResponderEnd,
    onResponderGrant: onResponderGrant,
    onResponderMove: onResponderMove,
    onResponderReject: onResponderReject,
    onResponderRelease: onResponderRelease,
    onResponderStart: onResponderStart,
    onResponderTerminate: onResponderTerminate,
    onResponderTerminationRequest: onResponderTerminationRequest,
    onScrollShouldSetResponder: onScrollShouldSetResponder,
    onScrollShouldSetResponderCapture: onScrollShouldSetResponderCapture,
    onSelectionChangeShouldSetResponder: onSelectionChangeShouldSetResponder,
    onSelectionChangeShouldSetResponderCapture: onSelectionChangeShouldSetResponderCapture,
    onStartShouldSetResponder: onStartShouldSetResponder,
    onStartShouldSetResponderCapture: onStartShouldSetResponderCapture
  });

  function handleClick(e) {
    if (onClick != null) {
      onClick(e);
    }

    if (onClick == null && onPress != null) {
      e.stopPropagation();
      onPress(e);
    }
  }

  var component = hasTextAncestor ? 'span' : 'div';
  var element = (0, _createElement.default)(component, {
    accessibilityLabel: accessibilityLabel,
    accessibilityLiveRegion: accessibilityLiveRegion,
    accessibilityRelationship: accessibilityRelationship,
    accessibilityRole: accessibilityRole,
    accessibilityState: accessibilityState,
    accessible: onPress != null ? true : null,
    children: children,
    classList: classList,
    // allow browsers to automatically infer the language writing direction
    dir: dir !== undefined ? dir : 'auto',
    importantForAccessibility: importantForAccessibility,
    nativeID: nativeID,
    onBlur: onBlur,
    onClick: handleClick,
    onContextMenu: onContextMenu,
    onFocus: onFocus,
    ref: setRef,
    style: style,
    testID: testID,
    // unstable
    onMouseDown: onMouseDown,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave,
    onMouseMove: onMouseMove,
    onMouseOver: onMouseOver,
    onMouseOut: onMouseOut,
    onMouseUp: onMouseUp,
    onTouchCancel: onTouchCancel,
    onTouchCancelCapture: onTouchCancelCapture,
    onTouchEnd: onTouchEnd,
    onTouchEndCapture: onTouchEndCapture,
    onTouchMove: onTouchMove,
    onTouchMoveCapture: onTouchMoveCapture,
    onTouchStart: onTouchStart,
    onTouchStartCapture: onTouchStartCapture,
    href: href,
    itemID: itemID,
    itemRef: itemRef,
    itemProp: itemProp,
    itemScope: itemScope,
    itemType: itemType,
    rel: rel,
    target: target,
    unstable_ariaSet: unstable_ariaSet,
    unstable_dataSet: unstable_dataSet
  });
  return hasTextAncestor ? element : _react.default.createElement(_TextAncestorContext.default.Provider, {
    value: true
  }, element);
});
Text.displayName = 'Text';

var classes = _css.default.create({
  text: {
    border: '0 solid black',
    boxSizing: 'border-box',
    color: 'black',
    display: 'inline',
    font: '14px System',
    margin: 0,
    padding: 0,
    whiteSpace: 'pre-wrap',
    wordWrap: 'break-word'
  },
  textHasAncestor: {
    color: 'inherit',
    font: 'inherit',
    whiteSpace: 'inherit'
  },
  textOneLine: {
    maxWidth: '100%',
    overflow: 'hidden',
    textOverflow: 'ellipsis',
    whiteSpace: 'nowrap'
  },
  // See #13
  textMultiLine: {
    display: '-webkit-box',
    maxWidth: '100%',
    overflow: 'hidden',
    textOverflow: 'ellipsis',
    WebkitBoxOrient: 'vertical'
  }
});

var styles = _StyleSheet.default.create({
  notSelectable: {
    userSelect: 'none'
  },
  pressable: {
    cursor: 'pointer'
  }
});

var _default = Text;
exports.default = _default;
module.exports = exports.default;