"use strict";

exports.__esModule = true;
exports.default = void 0;

var _createElement = _interopRequireDefault(require("../createElement"));

var _css = _interopRequireDefault(require("../StyleSheet/css"));

var _setAndForwardRef = _interopRequireDefault(require("../../modules/setAndForwardRef"));

var _useElementLayout = _interopRequireDefault(require("../../hooks/useElementLayout"));

var _usePlatformMethods = _interopRequireDefault(require("../../hooks/usePlatformMethods"));

var _useResponderEvents = _interopRequireDefault(require("../../hooks/useResponderEvents"));

var _StyleSheet = _interopRequireDefault(require("../StyleSheet"));

var _TextAncestorContext = _interopRequireDefault(require("../Text/TextAncestorContext"));

var _react = _interopRequireWildcard(require("react"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function _getRequireWildcardCache() { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) Nicolas Gallagher.
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
function createHitSlopElement(hitSlop) {
  var hitSlopStyle = {};

  for (var prop in hitSlop) {
    if (hitSlop.hasOwnProperty(prop)) {
      var value = hitSlop[prop];
      hitSlopStyle[prop] = value > 0 ? -1 * value : 0;
    }
  }

  return (0, _createElement.default)('span', {
    classList: [classes.hitSlop],
    style: hitSlopStyle
  });
}

var View = (0, _react.forwardRef)(function (props, ref) {
  var accessibilityLabel = props.accessibilityLabel,
      accessibilityLiveRegion = props.accessibilityLiveRegion,
      accessibilityRelationship = props.accessibilityRelationship,
      accessibilityRole = props.accessibilityRole,
      accessibilityState = props.accessibilityState,
      accessibilityValue = props.accessibilityValue,
      accessible = props.accessible,
      forwardedRef = props.forwardedRef,
      hitSlop = props.hitSlop,
      importantForAccessibility = props.importantForAccessibility,
      nativeID = props.nativeID,
      onBlur = props.onBlur,
      onClick = props.onClick,
      onContextMenu = props.onContextMenu,
      onFocus = props.onFocus,
      onLayout = props.onLayout,
      onMoveShouldSetResponder = props.onMoveShouldSetResponder,
      onMoveShouldSetResponderCapture = props.onMoveShouldSetResponderCapture,
      onResponderEnd = props.onResponderEnd,
      onResponderGrant = props.onResponderGrant,
      onResponderMove = props.onResponderMove,
      onResponderReject = props.onResponderReject,
      onResponderRelease = props.onResponderRelease,
      onResponderStart = props.onResponderStart,
      onResponderTerminate = props.onResponderTerminate,
      onResponderTerminationRequest = props.onResponderTerminationRequest,
      onScrollShouldSetResponder = props.onScrollShouldSetResponder,
      onScrollShouldSetResponderCapture = props.onScrollShouldSetResponderCapture,
      onSelectionChangeShouldSetResponder = props.onSelectionChangeShouldSetResponder,
      onSelectionChangeShouldSetResponderCapture = props.onSelectionChangeShouldSetResponderCapture,
      onStartShouldSetResponder = props.onStartShouldSetResponder,
      onStartShouldSetResponderCapture = props.onStartShouldSetResponderCapture,
      pointerEvents = props.pointerEvents,
      testID = props.testID,
      onKeyDown = props.onKeyDown,
      onKeyUp = props.onKeyUp,
      onMouseDown = props.onMouseDown,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      onMouseMove = props.onMouseMove,
      onMouseOver = props.onMouseOver,
      onMouseOut = props.onMouseOut,
      onMouseUp = props.onMouseUp,
      onScroll = props.onScroll,
      onTouchCancel = props.onTouchCancel,
      onTouchCancelCapture = props.onTouchCancelCapture,
      onTouchEnd = props.onTouchEnd,
      onTouchEndCapture = props.onTouchEndCapture,
      onTouchMove = props.onTouchMove,
      onTouchMoveCapture = props.onTouchMoveCapture,
      onTouchStart = props.onTouchStart,
      onTouchStartCapture = props.onTouchStartCapture,
      onWheel = props.onWheel,
      href = props.href,
      itemID = props.itemID,
      itemRef = props.itemRef,
      itemProp = props.itemProp,
      itemScope = props.itemScope,
      itemType = props.itemType,
      rel = props.rel,
      target = props.target,
      unstable_ariaSet = props.unstable_ariaSet,
      unstable_dataSet = props.unstable_dataSet;

  if (process.env.NODE_ENV !== 'production') {
    _react.default.Children.toArray(props.children).forEach(function (item) {
      if (typeof item === 'string') {
        console.error("Unexpected text node: " + item + ". A text node cannot be a child of a <View>.");
      }
    });
  }

  var hasTextAncestor = (0, _react.useContext)(_TextAncestorContext.default);
  var hostRef = (0, _react.useRef)(null);
  var setRef = (0, _setAndForwardRef.default)({
    getForwardedRef: function getForwardedRef() {
      return forwardedRef;
    },
    setLocalRef: function setLocalRef(c) {
      hostRef.current = c;
    }
  });
  var children = hitSlop ? _react.default.Children.toArray([createHitSlopElement(hitSlop), props.children]) : props.children;
  var classList = [classes.view];

  var style = _StyleSheet.default.compose(hasTextAncestor && styles.inline, props.style);

  (0, _useElementLayout.default)(hostRef, onLayout);
  (0, _usePlatformMethods.default)(hostRef, ref, classList, style);
  (0, _useResponderEvents.default)(hostRef, {
    onMoveShouldSetResponder: onMoveShouldSetResponder,
    onMoveShouldSetResponderCapture: onMoveShouldSetResponderCapture,
    onResponderEnd: onResponderEnd,
    onResponderGrant: onResponderGrant,
    onResponderMove: onResponderMove,
    onResponderReject: onResponderReject,
    onResponderRelease: onResponderRelease,
    onResponderStart: onResponderStart,
    onResponderTerminate: onResponderTerminate,
    onResponderTerminationRequest: onResponderTerminationRequest,
    onScrollShouldSetResponder: onScrollShouldSetResponder,
    onScrollShouldSetResponderCapture: onScrollShouldSetResponderCapture,
    onSelectionChangeShouldSetResponder: onSelectionChangeShouldSetResponder,
    onSelectionChangeShouldSetResponderCapture: onSelectionChangeShouldSetResponderCapture,
    onStartShouldSetResponder: onStartShouldSetResponder,
    onStartShouldSetResponderCapture: onStartShouldSetResponderCapture
  });
  return (0, _createElement.default)('div', {
    accessibilityLabel: accessibilityLabel,
    accessibilityLiveRegion: accessibilityLiveRegion,
    accessibilityRelationship: accessibilityRelationship,
    accessibilityRole: accessibilityRole,
    accessibilityState: accessibilityState,
    accessibilityValue: accessibilityValue,
    accessible: accessible,
    children: children,
    classList: classList,
    importantForAccessibility: importantForAccessibility,
    nativeID: nativeID,
    onBlur: onBlur,
    onClick: onClick,
    onContextMenu: onContextMenu,
    onFocus: onFocus,
    pointerEvents: pointerEvents,
    ref: setRef,
    style: style,
    testID: testID,
    // unstable
    onKeyDown: onKeyDown,
    onKeyUp: onKeyUp,
    onMouseDown: onMouseDown,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave,
    onMouseMove: onMouseMove,
    onMouseOver: onMouseOver,
    onMouseOut: onMouseOut,
    onMouseUp: onMouseUp,
    onScroll: onScroll,
    onTouchCancel: onTouchCancel,
    onTouchCancelCapture: onTouchCancelCapture,
    onTouchEnd: onTouchEnd,
    onTouchEndCapture: onTouchEndCapture,
    onTouchMove: onTouchMove,
    onTouchMoveCapture: onTouchMoveCapture,
    onTouchStart: onTouchStart,
    onTouchStartCapture: onTouchStartCapture,
    onWheel: onWheel,
    href: href,
    itemID: itemID,
    itemRef: itemRef,
    itemProp: itemProp,
    itemScope: itemScope,
    itemType: itemType,
    rel: rel,
    target: target,
    unstable_ariaSet: unstable_ariaSet,
    unstable_dataSet: unstable_dataSet
  });
});
View.displayName = 'View';

var classes = _css.default.create({
  view: {
    alignItems: 'stretch',
    border: '0 solid black',
    boxSizing: 'border-box',
    display: 'flex',
    flexBasis: 'auto',
    flexDirection: 'column',
    flexShrink: 0,
    margin: 0,
    minHeight: 0,
    minWidth: 0,
    padding: 0,
    position: 'relative',
    zIndex: 0
  },
  // this zIndex-ordering positions the hitSlop above the View but behind
  // its children
  hitSlop: {
    position: 'absolute',
    top: 0,
    left: 0,
    right: 0,
    bottom: 0,
    zIndex: -1
  }
});

var styles = _StyleSheet.default.create({
  inline: {
    display: 'inline-flex'
  }
});

var _default = View;
exports.default = _default;
module.exports = exports.default;