/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 * @format
 */
'use strict';

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import * as React from 'react';
import { useMemo, useState, useRef, useImperativeHandle } from 'react';
import usePressability from './usePressability';
import View from '../View';

/**
 * Component used to build display components that should respond to whether the
 * component is currently pressed or not.
 */
function Pressable(props, forwardedRef) {
  var _restProps$accessibil;

  var accessible = props.accessible,
      children = props.children,
      delayLongPress = props.delayLongPress,
      disabled = props.disabled,
      focusable = props.focusable,
      hitSlop = props.hitSlop,
      _onBlur = props.onBlur,
      _onFocus = props.onFocus,
      onLongPress = props.onLongPress,
      onPress = props.onPress,
      _onPressMove = props.onPressMove,
      _onPressIn = props.onPressIn,
      _onPressOut = props.onPressOut,
      style = props.style,
      testOnly_pressed = props.testOnly_pressed,
      restProps = _objectWithoutPropertiesLoose(props, ["accessible", "children", "delayLongPress", "disabled", "focusable", "hitSlop", "onBlur", "onFocus", "onLongPress", "onPress", "onPressMove", "onPressIn", "onPressOut", "style", "testOnly_pressed"]);

  var viewRef = useRef(null);
  useImperativeHandle(forwardedRef, function () {
    return viewRef.current;
  });

  var _usePressState = usePressState(testOnly_pressed === true),
      pressed = _usePressState[0],
      setPressed = _usePressState[1];

  var _useState = useState(false),
      focused = _useState[0],
      setFocused = _useState[1];

  var config = useMemo(function () {
    return {
      disabled: disabled,
      hitSlop: hitSlop,
      delayLongPress: delayLongPress,
      onBlur: function onBlur(event) {
        setFocused(false);

        if (_onBlur != null) {
          _onBlur(event);
        }
      },
      onFocus: function onFocus(event) {
        setFocused(true);

        if (_onFocus != null) {
          _onFocus(event);
        }
      },
      onLongPress: onLongPress,
      onPress: onPress,
      onPressIn: function onPressIn(event) {
        setPressed(true);

        if (_onPressIn != null) {
          _onPressIn(event);
        }
      },
      onPressMove: function onPressMove(event) {
        if (_onPressMove != null) {
          _onPressMove(event);
        }
      },
      onPressOut: function onPressOut(event) {
        setPressed(false);

        if (_onPressOut != null) {
          _onPressOut(event);
        }
      }
    };
  }, [delayLongPress, disabled, hitSlop, _onBlur, _onFocus, onLongPress, onPress, _onPressIn, _onPressMove, _onPressOut, setPressed]);
  var eventHandlers = usePressability(config);
  return React.createElement(View, _extends({}, restProps, {
    accessibilityRole: (_restProps$accessibil = restProps.accessibilityRole) !== null && _restProps$accessibil !== void 0 ? _restProps$accessibil : 'button',
    accessibilityState: _objectSpread({
      disabled: this.props.disabled
    }, this.props.accessibilityState),
    accessible: accessible !== false,
    focusable: focusable !== false,
    hitSlop: hitSlop,
    ref: viewRef,
    style: typeof style === 'function' ? style({
      focused: focused,
      pressed: pressed
    }) : style
  }, eventHandlers), typeof children === 'function' ? children({
    focused: focused,
    pressed: pressed
  }) : children);
}

function usePressState(forcePressed) {
  var _useState2 = useState(false),
      pressed = _useState2[0],
      setPressed = _useState2[1];

  return [pressed || forcePressed, setPressed];
}

var MemoedPressable = React.memo(React.forwardRef(Pressable));
MemoedPressable.displayName = 'Pressable';
export default MemoedPressable;