/**
 * Copyright (c) Nicolas Gallagher.
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
import createElement from '../createElement';
import css from '../StyleSheet/css';
import setAndForwardRef from '../../modules/setAndForwardRef';
import useElementLayout from '../../hooks/useElementLayout';
import usePlatformMethods from '../../hooks/usePlatformMethods';
import useResponderEvents from '../../hooks/useResponderEvents';
import React, { forwardRef, useContext, useRef } from 'react';
import StyleSheet from '../StyleSheet';
import TextAncestorContext from './TextAncestorContext';
var Text = forwardRef(function (props, ref) {
  var accessibilityLabel = props.accessibilityLabel,
      accessibilityLiveRegion = props.accessibilityLiveRegion,
      accessibilityRelationship = props.accessibilityRelationship,
      accessibilityRole = props.accessibilityRole,
      accessibilityState = props.accessibilityState,
      children = props.children,
      dir = props.dir,
      forwardedRef = props.forwardedRef,
      importantForAccessibility = props.importantForAccessibility,
      nativeID = props.nativeID,
      numberOfLines = props.numberOfLines,
      onBlur = props.onBlur,
      onClick = props.onClick,
      onContextMenu = props.onContextMenu,
      onFocus = props.onFocus,
      onLayout = props.onLayout,
      onPress = props.onPress,
      onMoveShouldSetResponder = props.onMoveShouldSetResponder,
      onMoveShouldSetResponderCapture = props.onMoveShouldSetResponderCapture,
      onResponderEnd = props.onResponderEnd,
      onResponderGrant = props.onResponderGrant,
      onResponderMove = props.onResponderMove,
      onResponderReject = props.onResponderReject,
      onResponderRelease = props.onResponderRelease,
      onResponderStart = props.onResponderStart,
      onResponderTerminate = props.onResponderTerminate,
      onResponderTerminationRequest = props.onResponderTerminationRequest,
      onScrollShouldSetResponder = props.onScrollShouldSetResponder,
      onScrollShouldSetResponderCapture = props.onScrollShouldSetResponderCapture,
      onSelectionChangeShouldSetResponder = props.onSelectionChangeShouldSetResponder,
      onSelectionChangeShouldSetResponderCapture = props.onSelectionChangeShouldSetResponderCapture,
      onStartShouldSetResponder = props.onStartShouldSetResponder,
      onStartShouldSetResponderCapture = props.onStartShouldSetResponderCapture,
      selectable = props.selectable,
      testID = props.testID,
      onMouseDown = props.onMouseDown,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      onMouseMove = props.onMouseMove,
      onMouseOver = props.onMouseOver,
      onMouseOut = props.onMouseOut,
      onMouseUp = props.onMouseUp,
      onTouchCancel = props.onTouchCancel,
      onTouchCancelCapture = props.onTouchCancelCapture,
      onTouchEnd = props.onTouchEnd,
      onTouchEndCapture = props.onTouchEndCapture,
      onTouchMove = props.onTouchMove,
      onTouchMoveCapture = props.onTouchMoveCapture,
      onTouchStart = props.onTouchStart,
      onTouchStartCapture = props.onTouchStartCapture,
      href = props.href,
      itemID = props.itemID,
      itemRef = props.itemRef,
      itemProp = props.itemProp,
      itemScope = props.itemScope,
      itemType = props.itemType,
      rel = props.rel,
      target = props.target,
      unstable_ariaSet = props.unstable_ariaSet,
      unstable_dataSet = props.unstable_dataSet;
  var hasTextAncestor = useContext(TextAncestorContext);
  var hostRef = useRef(null);
  var setRef = setAndForwardRef({
    getForwardedRef: function getForwardedRef() {
      return forwardedRef;
    },
    setLocalRef: function setLocalRef(c) {
      hostRef.current = c;
    }
  });
  var classList = [classes.text, hasTextAncestor === true && classes.textHasAncestor, numberOfLines === 1 && classes.textOneLine, numberOfLines != null && numberOfLines > 1 && classes.textMultiLine];
  var style = [props.style, numberOfLines != null && numberOfLines > 1 && {
    WebkitLineClamp: numberOfLines
  }, selectable === false && styles.notSelectable, onPress && styles.pressable];
  useElementLayout(hostRef, onLayout);
  usePlatformMethods(hostRef, ref, classList, style);
  useResponderEvents(hostRef, {
    onMoveShouldSetResponder: onMoveShouldSetResponder,
    onMoveShouldSetResponderCapture: onMoveShouldSetResponderCapture,
    onResponderEnd: onResponderEnd,
    onResponderGrant: onResponderGrant,
    onResponderMove: onResponderMove,
    onResponderReject: onResponderReject,
    onResponderRelease: onResponderRelease,
    onResponderStart: onResponderStart,
    onResponderTerminate: onResponderTerminate,
    onResponderTerminationRequest: onResponderTerminationRequest,
    onScrollShouldSetResponder: onScrollShouldSetResponder,
    onScrollShouldSetResponderCapture: onScrollShouldSetResponderCapture,
    onSelectionChangeShouldSetResponder: onSelectionChangeShouldSetResponder,
    onSelectionChangeShouldSetResponderCapture: onSelectionChangeShouldSetResponderCapture,
    onStartShouldSetResponder: onStartShouldSetResponder,
    onStartShouldSetResponderCapture: onStartShouldSetResponderCapture
  });

  function handleClick(e) {
    if (onClick != null) {
      onClick(e);
    }

    if (onClick == null && onPress != null) {
      e.stopPropagation();
      onPress(e);
    }
  }

  var component = hasTextAncestor ? 'span' : 'div';
  var element = createElement(component, {
    accessibilityLabel: accessibilityLabel,
    accessibilityLiveRegion: accessibilityLiveRegion,
    accessibilityRelationship: accessibilityRelationship,
    accessibilityRole: accessibilityRole,
    accessibilityState: accessibilityState,
    accessible: onPress != null ? true : null,
    children: children,
    classList: classList,
    // allow browsers to automatically infer the language writing direction
    dir: dir !== undefined ? dir : 'auto',
    importantForAccessibility: importantForAccessibility,
    nativeID: nativeID,
    onBlur: onBlur,
    onClick: handleClick,
    onContextMenu: onContextMenu,
    onFocus: onFocus,
    ref: setRef,
    style: style,
    testID: testID,
    // unstable
    onMouseDown: onMouseDown,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave,
    onMouseMove: onMouseMove,
    onMouseOver: onMouseOver,
    onMouseOut: onMouseOut,
    onMouseUp: onMouseUp,
    onTouchCancel: onTouchCancel,
    onTouchCancelCapture: onTouchCancelCapture,
    onTouchEnd: onTouchEnd,
    onTouchEndCapture: onTouchEndCapture,
    onTouchMove: onTouchMove,
    onTouchMoveCapture: onTouchMoveCapture,
    onTouchStart: onTouchStart,
    onTouchStartCapture: onTouchStartCapture,
    href: href,
    itemID: itemID,
    itemRef: itemRef,
    itemProp: itemProp,
    itemScope: itemScope,
    itemType: itemType,
    rel: rel,
    target: target,
    unstable_ariaSet: unstable_ariaSet,
    unstable_dataSet: unstable_dataSet
  });
  return hasTextAncestor ? element : React.createElement(TextAncestorContext.Provider, {
    value: true
  }, element);
});
Text.displayName = 'Text';
var classes = css.create({
  text: {
    border: '0 solid black',
    boxSizing: 'border-box',
    color: 'black',
    display: 'inline',
    font: '14px System',
    margin: 0,
    padding: 0,
    whiteSpace: 'pre-wrap',
    wordWrap: 'break-word'
  },
  textHasAncestor: {
    color: 'inherit',
    font: 'inherit',
    whiteSpace: 'inherit'
  },
  textOneLine: {
    maxWidth: '100%',
    overflow: 'hidden',
    textOverflow: 'ellipsis',
    whiteSpace: 'nowrap'
  },
  // See #13
  textMultiLine: {
    display: '-webkit-box',
    maxWidth: '100%',
    overflow: 'hidden',
    textOverflow: 'ellipsis',
    WebkitBoxOrient: 'vertical'
  }
});
var styles = StyleSheet.create({
  notSelectable: {
    userSelect: 'none'
  },
  pressable: {
    cursor: 'pointer'
  }
});
export default Text;