/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 * @format
 */
'use strict';

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

import * as React from 'react';
import Pressability from '../Pressable/Pressability';
import StyleSheet from '../StyleSheet';
import View from '../View';

/**
 * A wrapper for making views respond properly to touches.
 * On press down, the opacity of the wrapped view is decreased, which allows
 * the underlay color to show through, darkening or tinting the view.
 *
 * The underlay comes from wrapping the child in a new View, which can affect
 * layout, and sometimes cause unwanted visual artifacts if not used correctly,
 * for example if the backgroundColor of the wrapped view isn't explicitly set
 * to an opaque color.
 *
 * TouchableHighlight must have one child (not zero or more than one).
 * If you wish to have several child components, wrap them in a View.
 */
var TouchableHighlight =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(TouchableHighlight, _React$Component);

  function TouchableHighlight() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;
    _this._isMounted = false;
    _this.state = {
      pressability: new Pressability(_this._createPressabilityConfig()),
      extraStyles: _this.props.testOnly_pressed === true ? _this._createExtraStyles() : null
    };
    return _this;
  }

  var _proto = TouchableHighlight.prototype;

  _proto._createPressabilityConfig = function _createPressabilityConfig() {
    var _this2 = this;

    return {
      cancelable: !this.props.rejectResponderTermination,
      disabled: this.props.disabled,
      hitSlop: this.props.hitSlop,
      delayLongPress: this.props.delayLongPress,
      delayPressIn: this.props.delayPressIn,
      delayPressOut: this.props.delayPressOut,
      onBlur: function onBlur(event) {
        if (_this2.props.onBlur != null) {
          _this2.props.onBlur(event);
        }
      },
      onFocus: function onFocus(event) {
        if (_this2.props.onFocus != null) {
          _this2.props.onFocus(event);
        }
      },
      onLongPress: function onLongPress(event) {
        if (_this2.props.onLongPress != null) {
          _this2.props.onLongPress(event);
        }
      },
      onPress: function onPress(event) {
        if (_this2._hideTimeout != null) {
          clearTimeout(_this2._hideTimeout);
        }

        if (_this2.props.onPress != null) {
          _this2.props.onPress(event);
        }
      },
      onPressIn: function onPressIn(event) {
        if (_this2._hideTimeout != null) {
          clearTimeout(_this2._hideTimeout);
          _this2._hideTimeout = null;
        }

        _this2._showUnderlay();

        if (_this2.props.onPressIn != null) {
          _this2.props.onPressIn(event);
        }
      },
      onPressOut: function onPressOut(event) {
        if (_this2._hideTimeout == null) {
          _this2._hideUnderlay();
        }

        if (_this2.props.onPressOut != null) {
          _this2.props.onPressOut(event);
        }
      }
    };
  };

  _proto._createExtraStyles = function _createExtraStyles() {
    var _this$props$activeOpa;

    return {
      child: {
        opacity: (_this$props$activeOpa = this.props.activeOpacity) !== null && _this$props$activeOpa !== void 0 ? _this$props$activeOpa : 0.85
      },
      underlay: {
        backgroundColor: this.props.underlayColor === undefined ? 'black' : this.props.underlayColor
      }
    };
  };

  _proto._showUnderlay = function _showUnderlay() {
    var _this3 = this;

    if (!this._isMounted || !this._hasPressHandler()) {
      return;
    }

    this.setState(function () {
      return {
        extraStyles: _this3._createExtraStyles()
      };
    });

    if (this.props.onShowUnderlay != null) {
      this.props.onShowUnderlay();
    }
  };

  _proto._hideUnderlay = function _hideUnderlay() {
    if (this._hideTimeout != null) {
      clearTimeout(this._hideTimeout);
      this._hideTimeout = null;
    }

    if (this.props.testOnly_pressed === true) {
      return;
    }

    if (this._hasPressHandler()) {
      this.setState(function () {
        return {
          extraStyles: null
        };
      });

      if (this.props.onHideUnderlay != null) {
        this.props.onHideUnderlay();
      }
    }
  };

  _proto._hasPressHandler = function _hasPressHandler() {
    return this.props.onPress != null || this.props.onPressIn != null || this.props.onPressOut != null || this.props.onLongPress != null;
  };

  _proto.render = function render() {
    var child = React.Children.only(this.props.children);
    var eventHandlers = this.state.pressability.getEventHandlers();
    return React.createElement(View, _extends({
      accessibilityLabel: this.props.accessibilityLabel,
      accessibilityLiveRegion: this.props.accessibilityLiveRegion,
      accessibilityRole: this.props.accessibilityRole,
      accessibilityState: _objectSpread({
        disabled: this.props.disabled
      }, this.props.accessibilityState),
      accessibilityValue: this.props.accessibilityValue,
      accessible: this.props.accessible !== false,
      focusable: this.props.focusable !== false && this.props.onPress !== undefined,
      hitSlop: this.props.hitSlop,
      importantForAccessibility: this.props.importantForAccessibility,
      nativeID: this.props.nativeID,
      onLayout: this.props.onLayout,
      ref: this.props.hostRef,
      style: [styles.root, this.props.style, !this.props.disabled && styles.actionable, this.state.extraStyles && this.state.extraStyles.underlay],
      testID: this.props.testID
    }, eventHandlers), React.cloneElement(child, {
      style: StyleSheet.compose(child.props.style, this.state.extraStyles && this.state.extraStyles.child)
    }));
  };

  _proto.componentDidMount = function componentDidMount() {
    this._isMounted = true;
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    this.state.pressability.configure(this._createPressabilityConfig());
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this._isMounted = false;

    if (this._hideTimeout != null) {
      clearTimeout(this._hideTimeout);
    }

    this.state.pressability.reset();
  };

  return TouchableHighlight;
}(React.Component);

var TouchableHighlightWrapper = React.forwardRef(function (props, hostRef) {
  return React.createElement(TouchableHighlight, _extends({}, props, {
    hostRef: hostRef
  }));
});
TouchableHighlightWrapper.displayName = 'TouchableHighlight';
export default TouchableHighlightWrapper;
var styles = StyleSheet.create({
  root: {
    userSelect: 'none'
  },
  actionable: {
    cursor: 'pointer',
    touchAction: 'manipulation'
  }
});