/**
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 * @format
 */
'use strict';

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

import * as React from 'react';
import Pressability from '../Pressable/Pressability';
import StyleSheet from '../StyleSheet';
import View from '../View';

/**
 * A wrapper for making views respond properly to touches.
 * On press down, the opacity of the wrapped view is decreased, dimming it.
 *
 * Opacity is controlled by wrapping the children in an Animated.View, which is
 * added to the view hierarchy.  Be aware that this can affect layout.
 */
var TouchableOpacity =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(TouchableOpacity, _React$Component);

  function TouchableOpacity() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;
    _this.state = {
      duration: '0s',
      opacity: _this._getChildStyleOpacityWithDefault(),
      pressability: new Pressability(_this._createPressabilityConfig())
    };
    return _this;
  }

  var _proto = TouchableOpacity.prototype;

  _proto._createPressabilityConfig = function _createPressabilityConfig() {
    var _this2 = this;

    return {
      cancelable: !this.props.rejectResponderTermination,
      disabled: this.props.disabled,
      hitSlop: this.props.hitSlop,
      delayLongPress: this.props.delayLongPress,
      delayPressIn: this.props.delayPressIn,
      delayPressOut: this.props.delayPressOut,
      onBlur: function onBlur(event) {
        if (_this2.props.onBlur != null) {
          _this2.props.onBlur(event);
        }
      },
      onFocus: function onFocus(event) {
        if (_this2.props.onFocus != null) {
          _this2.props.onFocus(event);
        }
      },
      onLongPress: this.props.onLongPress,
      onPress: this.props.onPress,
      onPressIn: function onPressIn(event) {
        _this2._opacityActive(event.dispatchConfig.registrationName === 'onResponderGrant' ? 0 : 150);

        if (_this2.props.onPressIn != null) {
          _this2.props.onPressIn(event);
        }
      },
      onPressOut: function onPressOut(event) {
        _this2._opacityInactive(250);

        if (_this2.props.onPressOut != null) {
          _this2.props.onPressOut(event);
        }
      }
    };
  }
  /**
   * Animate the touchable to a new opacity.
   */
  ;

  _proto._setOpacityTo = function _setOpacityTo(value, duration) {
    this.setState(function () {
      return {
        opacity: value,
        duration: duration ? duration / 1000 + "s" : '0s'
      };
    });
  };

  _proto._opacityActive = function _opacityActive(duration) {
    var _this$props$activeOpa;

    this._setOpacityTo((_this$props$activeOpa = this.props.activeOpacity) !== null && _this$props$activeOpa !== void 0 ? _this$props$activeOpa : 0.2, duration);
  };

  _proto._opacityInactive = function _opacityInactive(duration) {
    this._setOpacityTo(this._getChildStyleOpacityWithDefault(), duration);
  };

  _proto._getChildStyleOpacityWithDefault = function _getChildStyleOpacityWithDefault() {
    var flatStyle = StyleSheet.flatten(this.props.style);
    var opacity = flatStyle != null ? flatStyle.opacity : null;
    return typeof opacity === 'number' ? opacity : 1;
  };

  _proto.render = function render() {
    var eventHandlers = this.state.pressability.getEventHandlers();
    return React.createElement(View, _extends({
      accessibilityLabel: this.props.accessibilityLabel,
      accessibilityLiveRegion: this.props.accessibilityLiveRegion,
      accessibilityRole: this.props.accessibilityRole,
      accessibilityState: _objectSpread({
        disabled: this.props.disabled
      }, this.props.accessibilityState),
      accessibilityValue: this.props.accessibilityValue,
      accessible: this.props.accessible !== false,
      focusable: this.props.focusable !== false && this.props.onPress !== undefined,
      hitSlop: this.props.hitSlop,
      importantForAccessibility: this.props.importantForAccessibility,
      nativeID: this.props.nativeID,
      onLayout: this.props.onLayout,
      ref: this.props.hostRef,
      style: [styles.root, !this.props.disabled && styles.actionable, this.props.style, {
        opacity: this.state.opacity,
        transitionDuration: this.state.duration
      }],
      testID: this.props.testID
    }, eventHandlers), this.props.children);
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    this.state.pressability.configure(this._createPressabilityConfig());

    if (this.props.disabled !== prevProps.disabled) {
      this._opacityInactive(250);
    }
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.state.pressability.reset();
  };

  return TouchableOpacity;
}(React.Component);

var TouchableOpacityWrapper = React.forwardRef(function (props, hostRef) {
  return React.createElement(TouchableOpacity, _extends({}, props, {
    hostRef: hostRef
  }));
});
TouchableOpacityWrapper.displayName = 'TouchableOpacity';
var styles = StyleSheet.create({
  root: {
    transitionProperty: 'opacity',
    transitionDuration: '0.15s',
    userSelect: 'none'
  },
  actionable: {
    cursor: 'pointer',
    touchAction: 'manipulation'
  }
});
export default TouchableOpacityWrapper;